#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include <mysql/mysql.h>
#include <bcm2835.h>

#define TEMPO_DELAY 250 //ms
#define DEBUG 1

#define HOST "localhost"
#define USER "root"
#define PASS "mysqllt38c"
#define DB "lt38c"
#define TB_W "pinwr"
#define TB_R "pinrd"

MYSQL* conn;

void logs (const char* mensagem)
{
	FILE* log = fopen("/tmp/ks0212gpio.log","a");
	if(log == NULL)
	{
        if(DEBUG)
            printf("Erro ao abrir arquivo de LOG\n");
		return;
	}
	fprintf(log,"%s",mensagem);
	fclose(log);
}

void inicializar_pinos()
{
    if(DEBUG)
        printf("Inicializando pinos\n");
	char buffer[512];
    if(DEBUG)
        printf("Pinos de saida: 1, 7, 8 e 25\n");
	
    bcm2835_gpio_fsel( 4, BCM2835_GPIO_FSEL_OUTP);	//wPi 07 <==> BCM 04
    bcm2835_gpio_fsel(22, BCM2835_GPIO_FSEL_OUTP);	//wPi 03 <==> BCM 22
    bcm2835_gpio_fsel( 6, BCM2835_GPIO_FSEL_OUTP);	//wPi 22 <==> BCM 06
    bcm2835_gpio_fsel(26, BCM2835_GPIO_FSEL_OUTP);	//wPi 25 <==> BCM 26
	
    if(DEBUG)
        printf("Pinos de entrada: 20 e 21\n");


    bcm2835_gpio_fsel(20, BCM2835_GPIO_FSEL_INPT);
    bcm2835_gpio_fsel(21, BCM2835_GPIO_FSEL_INPT);

    if(DEBUG)
        printf("Inicializacao de pinos concluída\n");
}

void consultaPinwr(){
    MYSQL_RES *resp;
    MYSQL_ROW linhas;
    char query[128]="SELECT * FROM pinwr WHERE sync IN (1,2);";
    char l[500];
    char update[500];
    unsigned char gpio;

    if (mysql_query(conn,query)) {
        if(DEBUG)
            printf("Erro: %s\n",mysql_error(conn));
        snprintf(l, sizeof(l),"Erro: %s\n",mysql_error(conn));
        logs(l);
    }            
    else{
        resp = mysql_store_result(conn);//recebe a consulta

        if (resp){ //se houver retorno                
            while ((linhas=mysql_fetch_row(resp)) != NULL){
		logs("---------------------------------PINWR------------------------------\n");
                if(DEBUG)
                    printf("Novo comando identificado --> ID = %s. PIN_%s = %s\n", linhas[0], linhas[2], linhas[1]);
                snprintf(l, sizeof(l),"Novo comando identificado --> ID = %s. PIN_%s = %s;", linhas[0], linhas[2], linhas[1]);
                logs(l);
                gpio = (unsigned char)atoi(linhas[2]);
                if(strcmp(linhas[1], "0") == 0){                            
                    if(DEBUG) printf("GPIO = [%d]\n", gpio);
                    bcm2835_gpio_write(gpio, LOW);
                    snprintf(update, sizeof(update),"UPDATE %s SET sync = 0, datah = now() WHERE pino = %s;", TB_W, linhas[2]);
                }                            
                else if(strcmp(linhas[1], "1") == 0){
                    if(DEBUG) printf("GPIO = [%d]\n", gpio);
                    bcm2835_gpio_write(gpio, HIGH);
                    snprintf(update, sizeof(update),"UPDATE %s SET sync = 0, datah = now() WHERE pino = %s;", TB_W, linhas[2]);
                }                            
                else if(strcmp(linhas[1], "2") == 0){ 
                    int tempoRestante = atoi(linhas[4]);

                    if(DEBUG)
                        printf("Comando com ID = %s sendo executado, tempo restante = %d.\n",linhas[0], tempoRestante);
                    snprintf(l, sizeof(l),"Comando com ID = %s sendo executado, tempo restante = %d.",linhas[0], tempoRestante);
                    logs(l);  

                    if(strcmp(linhas[3], "1") == 0){
                        bcm2835_gpio_write(gpio, HIGH);
                        snprintf(update, sizeof(update),"UPDATE %s SET sync = 2, datah = now() WHERE id = %s;", TB_W, linhas[0]);
                    }
                    else if(strcmp(linhas[3], "2") == 0){
                        tempoRestante = tempoRestante - TEMPO_DELAY;

                        if(tempoRestante > 0)
                            snprintf(update, sizeof(update),"UPDATE %s SET tempo = %d, datah = now() WHERE id = %s;", TB_W, tempoRestante, linhas[0]);
                        else{
                            bcm2835_gpio_write(gpio, LOW);
                            snprintf(update, sizeof(update),"UPDATE %s SET sync = 0, datah = now() WHERE id = %s;", TB_W, linhas[0]);
                        }                            
                    }                        
                }

                if(DEBUG)
                    printf("Comando PIN_%s = %s executado.\n", linhas[2], linhas[1]);
                snprintf(l, sizeof(l),"Comando PIN_%s = %s executado.\n", linhas[2], linhas[1]);
                logs(l);                

                if(mysql_query(conn, update)){                            
                    if(DEBUG)
                        printf("Erro: %s\n",mysql_error(conn));
                    snprintf(l, sizeof(l),"Erro: %s\n",mysql_error(conn));
                    logs(l);
                }
                else{
                    if(DEBUG)
                        printf("Atualizado tabela pinwr PIN_%s = %s e sync = 0 com sucesso!\n", linhas[2], linhas[1]);
                    snprintf(l, sizeof(l),"Atualizado tabela pinwr PIN_%s = %s sync = 0 com sucesso!\n", linhas[2], linhas[1]);
                    logs(l);
                }
            }                    
        }
        else{
            if(DEBUG)
                printf("Consulta: %s --> SEM RETORNO\n", query);
            snprintf(l, sizeof(l),"Consulta: %s --> SEM RETORNO\n", query);
            logs(l);
        }
    }
    // DESALOCAR VARIÁVEIS
    mysql_free_result(resp); 
}

void consultarPinrd(){
    MYSQL_RES *resp;
    MYSQL_FIELD *campos;
    MYSQL_ROW linhas;
    char query[128]="SELECT * FROM pinrd";
    char l[500];
    char update[500];
    int read_gpio;    
    unsigned char gpio;                   
    int read_ant;


    if (mysql_query(conn,query)) {
            if(DEBUG)
                printf("Erro: %s\n",mysql_error(conn));
            snprintf(l, sizeof(l),"Erro: %s\n",mysql_error(conn));
            logs(l);
    }            
    else{
        resp = mysql_store_result(conn);//recebe a consulta

        if (resp){ //se houver retorno                
            while ((linhas=mysql_fetch_row(resp)) != NULL){

                if(DEBUG)
                    printf("Leitura tabela pinrd, antes de ler GPIO, PIN_%s = %s\n", linhas[1], linhas[3]);
                snprintf(l, sizeof(l),"Leitura tabela pinr, antes de ler GPIO, PIN_%s = %s\n", linhas[1], linhas[3]);

		read_ant=(unsigned char)atoi(linhas[3]);
                gpio=(unsigned char)atoi(linhas[1]);                   
                read_gpio = bcm2835_gpio_lev(gpio);
                if(DEBUG)
                    printf("Leitura PIN_%s executada, valor lido: %i.\n", linhas[1], read_gpio);
                snprintf(l, sizeof(l),"Leitura PIN_%s executada, valor lido: %i.\n", linhas[1], read_gpio);

		if(read_ant != read_gpio){
		    logs("-----------------------PINRD-------------------\n");
		    logs(l);
		    }

                snprintf(update, sizeof(update),"UPDATE %s SET valor = %i, datah = now() WHERE id = %s;", TB_R, read_gpio, linhas[0]);                

                if(mysql_query(conn, update)){                            
                    if(DEBUG)
                        printf("Erro: %s\n",mysql_error(conn));
                    snprintf(l, sizeof(l),"Erro: %s\n",mysql_error(conn));
                    logs(l);
                }
                else{
                    if(DEBUG)
                        printf("Atualizado tabela pinrd PIN_%s = %i com sucesso!\n", linhas[1], read_gpio);
                    snprintf(l, sizeof(l),"Atualizado tabela pinrd PIN_%s = %i com sucesso!\n", linhas[1], read_gpio);
            	    if(read_ant != read_gpio)
				logs(l);
                }
            }                    
        }
        else{
            if(DEBUG)
                printf("Consulta: %s --> SEM RETORNO\n", query);
            snprintf(l, sizeof(l),"Consulta: %s --> SEM RETORNO\n", query);

        }
    } 
    // DESALOCAR VARIÁVEIS
    mysql_free_result(resp); 
}

void main(){
    while(true){  

        // INICIALIZA GPIO
        if(bcm2835_init()){
            if(DEBUG)
                printf("BCM inicializado\n");


            inicializar_pinos();
        }
        else{
            if(DEBUG)
                printf("Nao foi possivel incializar o BCM.\n");
            logs("Nao foi possivel incializar o BCM.\n");
        }

        // INICIALIZA MYSQL
        conn = mysql_init(NULL);

        // CONECTAR MYSQL
        if (mysql_real_connect(conn,HOST,USER,PASS,DB,0,NULL,0)){ //Tenta estabelecer uma conexao com o servidor

            if(DEBUG)
                printf("Conexao estabelecida.\n");

            if(DEBUG)
                printf("------------------------------------------PINWR----------------------------------\n");
            consultaPinwr(conn); 

            if(DEBUG)
        	 printf("------------------------------------------PINRD----------------------------------\n");
            consultarPinrd(conn);                                 
        }
        else{
            if(DEBUG)
                printf("Nao foi possivel realizar a conexao.\n");
            logs("Nao foi possivel realizar a conexao.\n");
        }

        if(!bcm2835_close())
        {
            if(DEBUG)
                printf("Erro: bcm2835 não conseguiu desligar\n");
            logs("Erro: bcm2835 não conseguiu desligar\n");
        }
        else
        {
            if(DEBUG)
                printf("bcm2835 desligado\n");
        }

        // DESCONECTAR
        mysql_close(conn);
        if(DEBUG)
                printf("Desconectado do banco de dados\n");
        delay(TEMPO_DELAY);
    }
}